import { computed, defineAsyncComponent, h } from 'vue'
import { isLayoutType } from '@cip/components/cip-form-layout/util'
import './content.less'
export default ({ selectId, onClick, onUpdateConfig, onDelete, onCopy, onSelectItem, element, showCopy = true }, { attrs }) => {
  // 获取渲染所需要的组件
  const getFormContentComponent = (type) => {
    return defineAsyncComponent(() => import(`./${type}`))
  }
  const getComponentType = (element) => {
    const { config: { type } } = element
    if (type === 'table') {
      return 'table'
    } else if (isLayoutType(type)) {
      return 'layout'
    } else {
      return 'item'
    }
  }

  const itemFieldKey = computed(() => {
    const fieldKey = formContentProps.fieldKey
    const otherKey = formContentProps.config.otherKey
    const fieldKeyTotalName = [fieldKey].concat(otherKey).filter(v => !!v).join('-')
    return fieldKeyTotalName
  })

  // 配置组件props
  const formContentProps = {
    isActive: selectId === element.id,
    fieldKey: element.key,
    selectId: selectId,
    config: { ...element.config, hideItem: false }, // 劫持 hideItem 强制修改为false //将背景色修改为红色
    showCopy,
    'onUpdate:config': onUpdateConfig,
    onClick,
    onDelete,
    onCopy,
    onSelectItem
  }
  const type = getComponentType(element)
  const TransactionLayer = (props, { slots, attrs }) => (
    <div
      {...attrs}
      class={[
        'form-content',
        `form-drawing__${type}`,
        formContentProps.config.class,
        {
          'is-active': formContentProps.isActive,
          'form-drawing--hidden': element.config.hideItem
        }
      ]}
      style={{ gridColumn: `span ${element.config.span || 1}` }}
    >
      <span
        class="right-top item-field-key"> {itemFieldKey.value}
      </span>
      <i class={'el-icon-rank show-focus handle-icon move-icon'}/>
      <div class="right-bottom show-focus">
        {formContentProps.showCopy && <i class="el-icon-document-copy handle-icon" onClick={(e) => {
          e.stopPropagation()
          formContentProps.onCopy(e)
        }}/>}
        <i
          class="el-icon-delete handle-icon"
          onClick={(e) => {
            e.stopPropagation()
            formContentProps.onDelete(e)
          }}/>
      </div>
      {slots.default()}
    </div>
  )
  return h(TransactionLayer, { ...attrs, element }, {
    default: () => h(getFormContentComponent(type), { ...formContentProps })
  })
}
