import { computed, readonly } from 'vue'
import { ElTag } from 'element-plus'
import { formInputViewProps, useFormView } from '@cip/components/helper/form-input'
import { getValueByTemplate, isObject } from '@cip/utils/util'

import './index.less'

export default {
  props: formInputViewProps,
  inheritAttrs: false,
  setup (props) {
    const { securityConfig } = useFormView(props)
    const innerValue = computed(() => {
      return [].concat(props.modelValue)
    })
    const formatter = (v) => {
      const { formatter } = securityConfig.value
      if (typeof formatter === 'function') return formatter(v, readonly(securityConfig.value))
      return v
    }

    return () => <div class={'cip-input-tag--view'}>
      {
        innerValue.value.map(v => <ElTag class='border-no'>
          {{
            default: () => {
              const valueTemplate = props.config.valueTemplate
              if (props.config.valueTemplate && isObject(v)) {
                return getValueByTemplate(valueTemplate, v)
              } else {
                return formatter(v)
              }
            }
          }}
        </ElTag>)
      }
    </div>
  }
}
